<?php
//
// pMachine Publishing Software
//
// Version 2.4 (Pro2.4.004)
//
// Copyright (c) 2003 - 2005 pMachine, Inc.
//
// THIS IS COPYRIGHTED SOFTWARE
//
// License holders may alter or modify this software for their own use, 
// but may NOT resell or redistribute the modified or derivative version 
// without prior written consent from pMachine.  Components from this 
// software may not be extracted and used in other programs without prior 
// written consent from pMachine.
//
// PLEASE READ THE LICENSE AGREEMENT
//



//-------------------------------------
// RSS Factory
//-------------------------------------    

class RSS_Factory {
                            
    var $generator          = "pMachine 2.4";
    var $rss_version        = "";
    var $rss_output         = "";
    var $item_count         = 0;
    var $channel_elements   = array();
    var $item_elements      = array();
    var $image_elements     = array();
    var $list_elements      = array();


    //-------------------------------------
    // Constructor
    //-------------------------------------    

    function RSS_Factory()
    {
    }
    // END
    

    //-------------------------------------
    // Set RSS Channel Elements
    //-------------------------------------    

    function set_channel_element($element = '', $value = '')
    {
        if ($element != '' AND $value != '')
            $this->channel_elements[$element] = $this->convert_xml_entities($value);
    }
    // END
    

    //-------------------------------------
    // Set RSS Image Elements
    //-------------------------------------    

    function set_image_element($element = '', $value = '')
    {
        if ($element != '' AND $value != '')
            $this->image_elements[$element] = $this->convert_xml_entities($value);
    }
    // END
    

    //-------------------------------------
    // Set RSS Item Elements
    //-------------------------------------    

    function set_item_element($element = '', $value = '')
    {
    
    
    
        if ($element == 'title')
            $this->item_count ++;

        if ($element != '' AND $value != '')
            $this->item_elements[$this->item_count -1][$element] = $this->convert_xml_entities($value);
    }
    // END
    
    
    //------------------------------------------
    // Set RDF List Elements (version 1.0 only)
    //------------------------------------------    

    function set_list_element($url)
    {
        if ($url != '')
            $this->list_elements[] = $url;
    }
    // END


    //-------------------------------------
    // Fetch RSS Channel Elements
    //-------------------------------------    

    function fetch_channel_elements()
    {
        if (count($this->channel_elements) >0)
        {
            $str = "";
            
            reset($this->channel_elements);
                    
            foreach($this->channel_elements as $key => $val) 
            {                    
                $str .= "<".$key.">".$this->convert_xml_entities($val)."</".$key.">\n";
            }
            
        return $str;
        }
    }
    // END
    

    //-------------------------------------
    // Fetch RSS Image Elements
    //-------------------------------------    

    function fetch_image_elements()
    {    
        if (count($this->image_elements) > 0)
        {    
            $str = "<image>\n";
        
            reset($this->image_elements);
                    
            foreach($this->image_elements as $key => $val) 
            {                    
                $str .= "<".$key.">".$this->convert_xml_entities($val)."</".$key.">\n";
            }
                
            $str .= "</image>\n";
                
        return $str;
        }
    }
    // END
    
    
    //-------------------------------------
    // Fetch RSS Item Elements
    //-------------------------------------    

    function fetch_item_elements()
    {    
        if (count($this->item_elements) > 0)
        {
            $str = '';

            reset($this->item_elements);
            
            $i = 0;
            
            foreach($this->item_elements as $key) 
            {
                $str .= "<item";
                
                if ($this->rss_version == "1.0"  AND isset($this->item_elements[$i]['link']))
                {
                    $str .= " rdf:about=\"".$this->convert_xml_entities($this->item_elements[$i]['link'])."\"";
                }
                
                $str .= ">\n";          
            
                foreach($key as $k => $v)
                $str .= "<".$k.">".$this->convert_xml_entities($v)."</".$k.">\n";
                
                $str .= "</item>\n";   
                
                $i++;
            }
               
        return $str;
        }
    }
    // END


    //-------------------------------------
    // Fetch RDF List Elements
    //-------------------------------------    

    function fetch_rdf_list_elements()
    {    
        if (isset($this->list_elements))
        {    
            $str  = "<items>\n";
            $str .= "<rdf:Seq>\n";
        
            reset($this->list_elements);
                    
            foreach($this->list_elements as $val) 
            {                    
                $str .= "<rdf:li resource=\"".$this->convert_xml_entities($val)."\" />\n";
            }

            $str .= "</rdf:Seq>\n";
            $str .= "</items>\n";
                
        return $str;
        }
    }
    // END


    //-------------------------------------
    // Build RSS File
    //-------------------------------------    

    function build_rss_file($ver = "")
    {     
        $versions = array('0.91', '0.92', '1.0', '2.0');
        
        $this->rss_version = ($ver != "") ? $ver : $this->rss_version;
        
        $this->rss_version = (!in_array($this->rss_version, $versions)) ? '0.91' : $this->rss_version;        
    
        switch($this->rss_version)
        {
            case "0.91" : $encoding = "UTF-8";
                break;
            case "0.92" : $encoding = "";
                break;
            case "1.0"  : $encoding = "";
                break;
            case "2.0"  : $encoding = "";
                break;
        }

        $encoding = ($encoding == "") ? "" : "encoding=\"".$encoding."\" ";
        
        
        $build  = "<"."?xml version=\"1.0\" ".$encoding."?".">\n";
        
        $build .= "<!-- Generator=\"".$this->generator."\" -->\n";

        $build .= ($this->rss_version == '1.0') ? $this->set_rss1_headers() : $this->set_rss_gen_headers();
    
        $build .= $this->fetch_item_elements();
        
        $build .= $this->rss_close();
    
        $this->rss_output = $build;
    }
    // END    


    //-----------------------------------------
    // Set RSS Version 0.91, 0.92, 2.0 Headers
    //-----------------------------------------    

    function set_rss_gen_headers()
    {
        $hdr  = "<rss version=\"".$this->rss_version."\">\n";    
        
        $hdr .= "<channel>\n";        
            
        $hdr .= $this->fetch_channel_elements();
        
        $hdr .= $this->fetch_image_elements();

        return $hdr;
    }
    // END
    

    //-------------------------------------
    // Set RSS Version 1.0 Headers
    //-------------------------------------    

    function set_rss1_headers()
    {
        $title = $this->convert_xml_entities($this->channel_elements['title']);
        $link = $this->convert_xml_entities($this->channel_elements['link']);
        $desc = $this->convert_xml_entities($this->channel_elements['description']);

        unset($this->channel_elements['title']);
        unset($this->channel_elements['link']);
        unset($this->channel_elements['description']);

        $hdr  = "<rdf:RDF xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns=\"http://purl.org/rss/1.0/\">\n";
        
        $hdr .= "<channel rdf:about=\"".$link."\">\n";
        
        $hdr .= "<title>".$title."</title>\n";

        $hdr .= "<link>".$link."</link>\n";
        
        $hdr .= "<description>".$desc."</description>\n";
        
       // $hdr .= $this->fetch_channel_elements();
        
        
        if (isset($this->image_elements['url']))
        $hdr .= "<image rdf:resource=\"".$this->image_elements['url']."\" />\n";    

        $hdr .= $this->fetch_rdf_list_elements();
        
        
        $hdr .= "</channel>\n";
        
        if (isset($this->image_elements['url']))
        {
            $hdr .= "<image rdf:about=\"".$this->image_elements['url']."\" >\n";    
            
            $hdr .= "<title>".$title."</title>\n";
            
            $hdr .= "<link>".$link."</link>\n";
            
            $hdr .= "<url>".$this->image_elements['url']."</url>\n";
        
            $hdr .= "</image>\n";            
        }
                
        return $hdr;
    }
    // END


    //-------------------------------------
    // RSS Closing Tag
    //-------------------------------------    

    function rss_close()
    {
        if ($this->rss_version == '1.0')
            return "</rdf:RDF>";
        else 
            return  "</channel>\n".
                    "</rss>";            
    }
    // END


    //--------------------------------------------------------
    // Show RSS File (For reference display only)
    //--------------------------------------------------------

    function echo_rss_file($file = '')
    {
        $out = $this->rss_output;
        
        $out = $this->convert_xml_entities($out);
        
        $out = str_replace("\n", "<br />", $out);
    
        echo $out;
    }
    // END


    //-------------------------------------
    // Write RSS File
    //-------------------------------------    

    function write_rss_file($file = '')
    {
    
        if (! $fp = @fopen($file, "w+"))
        {
            return false;
        }
	
	    flock($fp, LOCK_EX);
	    
        fputs($fp, $this->rss_output);
	    
	    flock($fp, LOCK_UN);
        
        fclose($fp);
    }
    // END



    //-------------------------------------
    // Convert reserved XML entities
    //-------------------------------------    

    function convert_xml_entities($str)
    {
        $temp = '848ff8if9a6fb627faccdbcce6';
        
        $str = preg_replace("/&#(\d+);/", "$temp\\1;", $str);
        $str = preg_replace("/&(\w+);/",  "$temp\\1;", $str);
    
        $ents = array(
                        "&"   =>    "&amp;",
                        "<"   =>    "&lt;",
                        ">"   =>    "&gt;",
                        "\""  =>    "&quot;",
                        "'"   =>    "&apos;",
                        "-"   =>    "&#45;"
                    );
    
        foreach ($ents as $k => $v)
        {
            $str = preg_replace("/".$k."/", $v, $str);
        }
            
        $str = preg_replace("/$temp(\d+);/","&#\\1;",$str);
        $str = preg_replace("/$temp(\w+);/","&\\1;", $str);
            
        return stripslashes($str);
    }    
    // END



    //-------------------------------------
    // Version Pull-down menu
    //-------------------------------------    

    function version_menu($default = '')
    {
        $version = array('0.91', '0.92', '1.0', '2.0');   
    
        $selected = '';
    
        $form = "<select name='version' class='smpulldown'>\n";
                
        foreach ($version as $val)
        {
            $selected = ($default == $val) ? ' selected' : '';
        
            $form .= "<option value='".$val."'".$selected.">".$val."</option>\n";
        }

        $form .= "</select>\n";

        return $form;
    }
    // END



    //-------------------------------------
    // Language Pull-down menu
    //-------------------------------------    

    function language_menu($default = '')
    {
        $languages = $this->get_languages();    
    
        $selected = '';
    
        $form = "<select name='rss_language' class='smpulldown'>\n";
                
        foreach ($languages as $key => $val)
        {
            $selected = ($default == $key) ? ' selected' : '';
        
            $form .= "<option value='".$key."'".$selected.">".$val."</option>\n";            
        }

        $form .= "</select>\n";

        return $form;
    }
    // END


    //-------------------------------------
    // Assign Language Array
    //-------------------------------------    

    function get_languages()
    {
        return array(   'af'     =>  'Afrikaans',
                        'sq '    =>  'Albanian',
                        'eu'     =>  'Basque',
                        'be'     =>  'Belarusian',
                        'bg'     =>  'Bulgarian',
                        'ca'     =>  'Catalan',
                        'zh-cn'  =>  'Chinese (Simplified)',
                        'zh-tw'  =>  'Chinese (Traditional)',
                        'hr'     =>  'Croatian',
                        'cs'     =>  'Czech',
                        'da'     =>  'Danish',
                        'nl'     =>  'Dutch',
                        'nl-be'  =>  'Dutch (Belgium)',
                        'nl-nl'  =>  'Dutch (Netherlands)',
                        'en'     =>  'English',
                        'en-au'  =>  'English (Australia)',
                        'en-bz'  =>  'English (Belize)',
                        'en-ca'  =>  'English (Canada)',
                        'en-ie'  =>  'English (Ireland)',
                        'en-jm'  =>  'English (Jamaica)',
                        'en-nz'  =>  'English (New Zealand)',
                        'en-ph'  =>  'English (Phillipines)',
                        'en-za'  =>  'English (South Africa)',
                        'en-tt'  =>  'English (Trinidad)',
                        'en-gb'  =>  'English (United Kingdom)',
                        'en-us'  =>  'English (United States)',
                        'en-zw'  =>  'English (Zimbabwe)',
                        'fo'     =>  'Faeroese',
                        'fi'     =>  'Finnish',
                        'fr'     =>  'French',
                        'fr-be'  =>  'French (Belgium)',
                        'fr-ca'  =>  'French (Canada)',
                        'fr-fr'  =>  'French (France)',
                        'fr-lu'  =>  'French (Luxembourg)',
                        'fr-mc'  =>  'French (Monaco)',
                        'fr-ch'  =>  'French (Switzerland)',
                        'gl'     =>  'Galician',
                        'gd'     =>  'Gaelic',
                        'de'     =>  'German',
                        'de-at'  =>  'German (Austria)',
                        'de-de'  =>  'German (Germany)',
                        'de-li'  =>  'German (Liechtenstein)',
                        'de-lu'  =>  'German (Luxembourg)',
                        'de-ch'  =>  'German (Switzerland)',
                        'el'     =>  'Greek',
                        'hu'     =>  'Hungarian',
                        'is'     =>  'Icelandic',
                        'in'     =>  'Indonesian',
                        'ga'     =>  'Irish',
                        'it'     =>  'Italian',
                        'it-it'  =>  'Italian (Italy)',
                        'it-ch'  =>  'Italian (Switzerland)',
                        'ja'     =>  'Japanese',
                        'ko'     =>  'Korean',
                        'mk'     =>  'Macedonian',
                        'no'     =>  'Norwegian',
                        'pl'     =>  'Polish',
                        'pt'     =>  'Portuguese',
                        'pt-br'  =>  'Portuguese (Brazil)',
                        'pt-pt'  =>  'Portuguese (Portugal)',
                        'ro'     =>  'Romanian',
                        'ro-mo'  =>  'Romanian (Moldova)',
                        'ro-ro'  =>  'Romanian (Romania)',
                        'ru'     =>  'Russian',
                        'ru-mo'  =>  'Russian (Moldova)',
                        'ru-ru'  =>  'Russian (Russia)',
                        'sr'     =>  'Serbian',
                        'sk'     =>  'Slovak',
                        'sl'     =>  'Slovenian',
                        'es'     =>  'Spanish',
                        'es-ar'  =>  'Spanish (Argentina)',
                        'es-bo'  =>  'Spanish (Bolivia)',
                        'es-cl'  =>  'Spanish (Chile)',
                        'es-co'  =>  'Spanish (Colombia)',
                        'es-cr'  =>  'Spanish (Costa Rica)',
                        'es-do'  =>  'Spanish (Dominican Republic)',
                        'es-ec'  =>  'Spanish (Ecuador)',
                        'es-sv'  =>  'Spanish (El Salvador)',
                        'es-gt'  =>  'Spanish (Guatemala)',
                        'es-hn'  =>  'Spanish (Honduras)',
                        'es-mx'  =>  'Spanish (Mexico)',
                        'es-ni'  =>  'Spanish (Nicaragua)',
                        'es-pa'  =>  'Spanish (Panama)',
                        'es-py'  =>  'Spanish (Paraguay)',
                        'es-pe'  =>  'Spanish (Peru)',
                        'es-pr'  =>  'Spanish (Puerto Rico)',
                        'es-es'  =>  'Spanish (Spain)',
                        'es-uy'  =>  'Spanish (Uruguay)',
                        'es-ve'  =>  'Spanish (Venezuela)',
                        'sv'     =>  'Swedish',
                        'sv-fi'  =>  'Swedish (Finland)',
                        'sv-se'  =>  'Swedish (Sweden)',
                        'tr'     =>  'Turkish',
                        'uk'     =>  'Ukranian',
                        );
    }
    // END

}
// END CLASS
?>